(***********************************************************************)
(*                                                                     *)
(*                              Ref-image                              *)
(*                                                                     *)
(*                  Image Segmentation by Refinement                   *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2009-2013 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: configuration.mli,v 1.31 2013-10-29 14:18:32 fclement Exp $ *)

open Segmentation_types;;
open Configuration_types;;

(*val digest : configuration Utype_digest.t;;*)

(** {6 History functor interface} *)

val print : t Format_printer.printer;;

val cold_set : unit -> unit;;
val warm_set : t -> unit;;
val get : unit -> t;;

val phase_dimension : Ucommand_option.phase_name;;

(** {6 Specific shortcuts} *)

val get_dimension : unit -> dimension;;
val get_dimension_management : unit -> dimension_management;;
val get_cutting_strategy : unit -> cutting_strategy;;
val get_stopping_criterion : unit -> stopping_criterion;;
val get_image_width : unit -> width;;
val get_image_height : unit -> height;;

(** {6 Program configuration} *)

val get_formatter : unit -> Format.formatter;;
val get_report_allocated_memory : unit -> bool;;
val get_incremental_mode : unit -> bool;;
val get_graphics_mode : unit -> graphics_mode;;
val get_graphics_window_min_size : unit -> size;;
val get_graphics_window_max_size : unit -> size;;
val get_graphics_gradient_threshold : unit -> Unatural.t;;
val get_final_image_file_name : unit -> Ufilename.t;;

(** {6 Monitoring} *)

val report : Upervasives.message -> bool;;
val report_action : (unit -> Upervasives.message) -> bool;;
val debug :
  Upervasives.function_name -> Upervasives.message -> bool
;;
val debug_action :
  Upervasives.function_name -> (unit -> Upervasives.message) -> bool
;;
val fatal_error :
  Upervasives.function_name -> Upervasives.message -> int -> 'a
;;

val print_ident_value : string -> ('a, 'a) Format_printer.printer_lifter;;

val print_report :
  'a Format_printer.printer -> Format.formatter -> 'a -> bool
;;
val print_debug :
  Upervasives.function_name -> string ->
  'a Format_printer.printer -> Format.formatter -> 'a -> bool
;;
val print_debug_no_arg :
  Upervasives.function_name ->
  (Format.formatter -> unit) -> Format.formatter -> bool
;;
