(***********************************************************************)
(*                                                                     *)
(*                              Ref-image                              *)
(*                                                                     *)
(*                  Image Segmentation by Refinement                   *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2009-2013 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: configuration_types.mli,v 1.17 2013-09-30 11:34:50 fclement Exp $ *)

open Segmentation_types;;

type dimension = Unatural.t;;

type cutting_choice =
  | All_cuttings
  | Best_cutting

and component_choice =
  | All_components
  | Best_component
;;

type dimension_management = cutting_choice * component_choice;;
(**
  Vector =                  (Best_cutting, All_components)
  Best_component_only =     (Best_cutting, Best_component)
  Best_component_for_each = (All_cuttings, Best_component)
  Combine_best_components = (All_cuttings, All_components)
*)

type stopping_criterion =
  {
   max_iteration : History_types.state_number;
   min_best_scalar_indicator : float;
   min_scalar_relative_error : float;
  }
;;

type t =
  {
   dimension : dimension;
   dimension_management : dimension_management;
   cutting_strategy : cutting_strategy;
   stopping_criterion : stopping_criterion;
   image_width : width;
   image_height : height;
  }
;;

type quantity_choice =
  | Param_quantity
  | All_quantities

and channel_choice =
  | Compound_channel
  | All_channels
;;

type graphics_kind = quantity_choice * channel_choice;;
(**
  Minimal_graphics   = (Param_quantity, Compound_channel)
  Parameter_graphics = (Param_quantity, All_channels)
  Compound_graphics  = (All_quantities, Compound_channel)
  Full_graphics      = (All_quantities, All_channels)
*)

type graphics_mode =
  {
   on : bool;
   kind : graphics_kind option;
  }
;;

type program_configuration =
  {
   formatter : Format.formatter;
   report_allocated_memory : bool;
   incremental_mode : bool;
   graphics_mode : graphics_mode;
   graphics_window_min_size : size;
   graphics_window_max_size : size;
   graphics_gradient_threshold : Unatural.t;
   final_image_file_name : Ufilename.t;
  }
;;
