(***********************************************************************)
(*                                                                     *)
(*                              Ref-image                              *)
(*                                                                     *)
(*                  Image Segmentation by Refinement                   *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2009-2013 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: vector.ml,v 1.14 2013-12-02 15:28:38 fclement Exp $ *)

module Nat = Unatural;;

open Vector_types_spec;;

module Make =
  functor (D : DIMENSION) ->
struct
  include D;;
  
  module Vt = Vector_types.Make (D);;
  
  type 'a t = 'a Vt.t;;
  
  (*let gen_digest = Utype_digest.gen_unary_digest Vector_digest.md5;;
  let digest = gen_digest "vector";;*)
  
  let to_array = Vt.of_vector
  and of_array = Vt.to_vector
  ;;
  
  external unsafe_of_array : 'a array -> 'a t = "%identity";;
  
  let get v i = Array.unsafe_get (to_array v) (Nat.to_int i);;
  
  let make x = unsafe_of_array (Uarray.make (get_dim ()) x);;
  let init f =
    let ff i = f (Nat.of_int i) in
    unsafe_of_array (Uarray.init (get_dim ()) ff)
  ;;
  
  let to_list v = Array.to_list (to_array v);;
  let of_list l = of_array (Array.of_list l);;
  let iter f v = Array.iter f (to_array v);;
  let map f v = unsafe_of_array (Array.map f (to_array v));;
  let iter_i f v =
    let ff i = f (Nat.of_int i) in
    Array.iteri ff (to_array v)
  ;;
  let map_i f v =
    let ff i = f (Nat.of_int i) in
    unsafe_of_array (Array.mapi ff (to_array v))
  ;;
  let iter_rails fv v = Uarray.iter_rails (to_array fv) (to_array v);;
  let fold_left f accu v = Array.fold_left f accu (to_array v);;
  let fold_right f v = Array.fold_right f (to_array v);;
  let sort cmp v =
    let a = to_array v in
    Array.sort cmp a;
    unsafe_of_array a
  ;;
  
  let fold_left_i f accu v =
    let ff accu i = f accu (Nat.of_int i) in
    Uarray.fold_left_i ff accu (to_array v)
  ;;
  let fold_right_i f v =
    let ff i accu = f (Nat.of_int i) accu in
    Uarray.fold_right_i ff (to_array v)
  ;;
  let iter2 f v1 v2 = Uarray.iter2 f (to_array v1) (to_array v2);;
  let map2 f v1 v2 =
    unsafe_of_array (Uarray.map2 f (to_array v1) (to_array v2))
  ;;
  let iter_i2 f v1 v2 =
    let ff i = f (Nat.of_int i) in
    Uarray.iter_i2 ff (to_array v1) (to_array v2)
  ;;
  let map_i2 f v1 v2 =
    let ff i = f (Nat.of_int i) in
    unsafe_of_array (Uarray.map_i2 ff (to_array v1) (to_array v2))
  ;;
  let fold_left2 f accu v1 v2 =
    Uarray.fold_left2 f accu (to_array v1) (to_array v2)
  ;;
  let fold_right2 f v1 v2 = Uarray.fold_right2 f (to_array v1) (to_array v2);;
  let iter3 f v1 v2 v3 =
    Uarray.iter3 f (to_array v1) (to_array v2) (to_array v3)
  ;;
  let map3 f v1 v2 v3 =
    unsafe_of_array (Uarray.map3 f (to_array v1) (to_array v2) (to_array v3))
  ;;
  let iter4 f v1 v2 v3 v4 =
    Uarray.iter4 f (to_array v1) (to_array v2) (to_array v3) (to_array v4)
  ;;
  let map4 f v1 v2 v3 v4 =
    unsafe_of_array
      (Uarray.map4 f (to_array v1) (to_array v2) (to_array v3) (to_array v4))
  ;;
  let for_all p v = Uarray.for_all p (to_array v);;
  let exists p v = Uarray.exists p (to_array v);;
  let find p v = Uarray.find p (to_array v);;
  let find_all p v = Uarray.find_all p (to_array v);;
  
  let max_assoc v =
    let l = Array.to_list (to_array v) in
    Ulist.max_assoc l
  ;;
  
  let print p_a ppf v = Lib_print.print_Array p_a ppf (to_array v);;
end
;;

module Dim1 =
struct
  let get_dim () = Nat._1_;;
end
;;

module Dim2 =
struct
  let get_dim () = Nat._2_;;
end
;;

module Dim3 =
struct
  let get_dim () = Nat._3_;;
end
;;

module Dim4 =
struct
  let get_dim () = Nat._4_;;
end
;;

module V1 = Make (Dim1);;
module V2 = Make (Dim2);;
module V3 = Make (Dim3);;
module V4 = Make (Dim4);;
