(***********************************************************************)
(*                                                                     *)
(*                                Ulib                                 *)
(*                                                                     *)
(*                   OCaml user's additional library                   *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2004-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: ugraph.mli,v 1.2 2014/01/10 15:28:04 fclement Exp $ *)

(** {3 Graphs of functions} *)

type 'a index = 'a Unonempty_set.index
and 'a uindex = 'a Unonempty_set.uindex
;;

type 'a domain = 'a Unonempty_set.t;;

type 'a sub_domain = 'a Unonempty_set.nonempty_subset;;

type 'a images =
  | Constant of 'a
  | Images of 'a array
;;

type ('a, 'b) t = private
  {
   domain : 'a domain;
   images : 'b images;
  }
;;

val same : ('a, 'b) t -> ('a, 'b) t -> bool;;

(** {6 Construction functions} *)

val constant : 'a domain -> 'b -> ('a, 'b) t;;
val from_array : 'a domain -> 'b array -> ('a, 'b) t;;

val from_fun : 'a domain -> ('a index -> 'a uindex -> 'b) -> ('a, 'b) t;;

val of_matrix : 'a Umatrix.t -> (int * int, 'a) t;;

(** {6 Functions on graphs} *)

val image : ('a, 'b) t -> 'a index -> 'b;;

val restriction : ('a, 'b) t -> 'a sub_domain -> ('a, 'b) t;;

(** {6 Iterators} *)

val map_images : ('b -> 'c) -> ('a, 'b) t -> ('a, 'c) t;;
